from armstrong.utils.backends import GenericBackend

RICH_TEXT_BACKEND = GenericBackend('ARMSTRONG_HATBAND_RICHTEXTEDITOR',
        defaults="armstrong.hatband.widgets.ckeditor.CKEditorWidget")
#RichTextWidget = RICH_TEXT_BACKEND.get_backend


class RichTextWidget(object):
    def __new__(cls, *args, **kwargs):
        return RICH_TEXT_BACKEND.get_backend(*args, **kwargs)



from .._utils import *
from ...widgets import CKEditorWidget, RichTextWidget
from django.forms.widgets import Textarea
from django.conf import settings

TEST_SETTINGS = {
    "ARMSTRONG_HATBAND_RICHTEXTEDITOR": "django.forms.widgets.Textarea",
}


class RichTextFieldTestCase(HatbandTestCase):

    def test_default(self):
        ckeditor = CKEditorWidget()
        widget = RichTextWidget()
        ckeditor_out = str(ckeditor.render('widget', ''))
        widget_out = str(widget.render('widget', ''))
        self.assertEquals(ckeditor_out, widget_out)

    def test_non_default_setting(self):
        with self.settings(**TEST_SETTINGS):
            textarea = Textarea()
            widget = RichTextWidget()
            textarea_out = str(textarea.render('widget', ''))
            widget_out = str(widget.render('widget', ''))
            self.assertEquals(textarea_out, widget_out)

    def test_kwargs_passthrough(self):
        attrs = {'cols': 50, 'class': 'something'}
        ckeditor = CKEditorWidget(attrs=attrs)
        widget = RichTextWidget(attrs=attrs)
        ckeditor_out = str(ckeditor.render('widget', ''))
        widget_out = str(widget.render('widget', ''))
        self.assertEquals(ckeditor_out, widget_out)



from django import forms
from django.conf import settings
from django.db import models

from . import widgets

RICH_TEXT_DBFIELD_OVERRIDES = {
    models.TextField: {'widget': widgets.RichTextWidget},
}


class BackboneFormMixin(object):
    if getattr(settings, "ARMSTRONG_ADMIN_PROVIDE_STATIC", True):
        class Media:
            js = (
                    'hatband/js/jquery-1.6.2.min.js',
                    'hatband/js/underscore.js',
                    'hatband/js/backbone.js',
                    'hatband/js/backbone-inline-base.js')


class OrderableGenericKeyLookupForm(BackboneFormMixin, forms.ModelForm):
    class Meta:
        widgets = {
            "content_type": forms.HiddenInput(),
            "object_id": widgets.GenericKeyWidget(),
            "order": forms.HiddenInput(),
        }



from django.conf import settings
from .._utils import *
from ...widgets import CKEditorWidget


class CKEditorTestCase(HatbandTestCase):

    def test_render(self):
        widget = CKEditorWidget()
        htmlout = str(widget.render('widget', ''))
        self.assertTrue('class="ckeditor"' in htmlout)

    def test_render_with_additional_classes(self):
        widget = CKEditorWidget(attrs={'class': 'something'})
        htmlout = widget.render('widget', '')
        self.assertTrue('class="something ckeditor"' in htmlout)



from armstrong import hatband
from . import models
from django.db.models import TextField
from django.forms.widgets import TextInput


class ArticleAdmin(hatband.ModelAdmin):

    class Meta:
        model = models.TestArticle


class ArticleOverrideAdmin(hatband.ModelAdmin):

    formfield_overrides = {
        TextField: {'widget': TextInput},
    }

    class Meta:
        model = models.TestArticle


class ArticleTabularInline(hatband.TabularInline):

    class Meta:
        model = models.TestArticle


class ArticleStackedInline(hatband.StackedInline):

    class Meta:
        model = models.TestArticle


class CategoryAdminTabbed(hatband.ModelAdmin):

    inlines = ArticleTabularInline

    class Meta:
        model = models.TestCategory


class CategoryAdminStacked(hatband.ModelAdmin):

    inlines = ArticleStackedInline

    class Meta:
        model = models.TestCategory



from armstrong.core.arm_layout.utils import render_model
from django.conf import settings
from django.conf.urls.defaults import patterns
from django.conf.urls.defaults import url
from django.contrib.contenttypes.models import ContentType
from django.db.models import Q
from django.http import HttpResponse
from django.http import HttpResponseBadRequest

from ..http import JsonResponse

EXCLUDED_MODELS_FROM_FACETS = getattr(settings,
    "ARMSTRONG_EXCLUDED_MODELS_FROM_FACETS", [
        ("arm_wells", "welltype"),

        # remove this next line if we decide to support this
        ("arm_wells", "well"),
    ])

EXCLUDED_APPS_FROM_FACETS = getattr(settings,
    "ARMSTRONG_EXCLUDED_APPS_FROM_FACETS", [
        "admin", "auth", "contenttype", "reversion", "sessions", "sites",
    ])


class ArmstrongBaseMixin(object):
    url_prefix = "armstrong"


class GenericKeyFacetsMixin(ArmstrongBaseMixin):
    url = "search/generickey/facets/"

    def get_urls(self):
        urlpatterns = patterns('',
            url(r"^%s/%s$" % (self.url_prefix, self.url),
                self.admin_view(self.generic_key_facets),
                name="generic_key_facets",
            )
        )
        return urlpatterns + super(GenericKeyFacetsMixin, self).get_urls()

    def generic_key_facets(self, request):
        """Find all available facets/Models for VisualSearch"""

        excluded_apps = Q(app_label__in=EXCLUDED_APPS_FROM_FACETS)
        excluded_models = Q()
        for app_label, model in EXCLUDED_MODELS_FROM_FACETS:
            excluded_models = excluded_models | Q(app_label=app_label,
                    model=model)
        values = "model", "app_label", "id"
        content_types = ContentType.objects.values_list(*values) \
                .exclude(excluded_apps | excluded_models)
        return JsonResponse(dict([(str(a), {"app_label": str(b), "id": str(c)})
                for a, b, c in content_types]))


class ModelSearchBackfillMixin(object):
    slug = "search"
    view_name_template = "%s_%s_search"

    def get_urls(self):
        # TODO: Attempt
        model_urls = []
        for model, model_admin in self._registry.iteritems():
            model_urls.append(
                    url(r"^%s/%s/%s/$" % (
                            model._meta.app_label,
                            model._meta.module_name,
                            self.slug),
                    self.admin_view(self.generic_key_modelsearch),
                    kwargs={'app_label': model._meta.app_label,
                            'model_name': model._meta.module_name},
                    name=self.view_name_template % (
                            model._meta.app_label,
                            model._meta.module_name))
                )

        return patterns('', *model_urls) \
                + super(ModelSearchBackfillMixin, self).get_urls()

    def generic_key_modelsearch(self, request, app_label, model_name):
        """
        Find instances for the requested model and return them as JSON.
        # TODO: add test coverage for this
        """
        content_type = ContentType.objects.get(app_label=app_label,
                model=model_name)

        model = content_type.model_class()
        model_admin = self._registry[model].__class__(model, self)
        model_admin.change_list_template = "admin/hatband/change_list.json"
        return model_admin.changelist_view(request)


class ModelPreviewMixin(ArmstrongBaseMixin):
    """
    Provides a way to render a model's preview display
    """
    slug = "render_model_preview"

    def get_urls(self):
        urlpatterns = patterns('',
            url(r"^%s/%s/$" % (self.url_prefix, self.slug),
                self.admin_view(self.render_model_preview),
                name="render_model_preview"
            )
        )
        return urlpatterns + super(ModelPreviewMixin, self).get_urls()

    def render_model_preview(self, request):
        type = ContentType.objects.get(pk=request.GET["content_type"])
        model = type.model_class().objects.get(pk=request.GET["object_id"])
        template = request.GET.get("template", "preview")
        return HttpResponse(render_model(model, template))


class TypeAndModelQueryMixin(ArmstrongBaseMixin):
    type_and_model_to_query_url = "search/type_and_model_to_query/"

    def get_urls(self):
        urlpatterns = patterns('',
            url(r"^%s/%s$" % (self.url_prefix,
                              self.type_and_model_to_query_url),
                self.admin_view(self.type_and_model_to_query),
                name="type_and_model_to_query",
            )
        )
        return urlpatterns + super(TypeAndModelQueryMixin, self).get_urls()

    def type_and_model_to_query(self, request):
        """
        Return JSON for an individual Model instance

        If the required parameters are wrong, return 400 Bad Request
        If the parameters are correct but there is no data, return empty JSON

        """
        try:
            content_type = ContentType.objects.get(
                    pk=request.GET.get("content_type_id"))
        except ContentType.DoesNotExist:
            return HttpResponseBadRequest()

        try:
            model = content_type.model_class()
            result = model.objects.get(pk=request.GET["object_id"])
        except KeyError:
            return HttpResponseBadRequest()
        except model.DoesNotExist:
            data = ""
        else:
            data = '%s: "%d: %s"' % (content_type.model, result.pk, result)

        return JsonResponse({"query": data})



from django.conf import settings
from django.core.urlresolvers import reverse
from django.forms import Widget
from django.template.loader import render_to_string

from ..utils import static_url


class GenericKeyWidget(Widget):
    template = "admin/hatband/widgets/generickey.html"

    if getattr(settings, "ARMSTRONG_ADMIN_PROVIDE_STATIC", True):
        class Media:
            js = (static_url("visualsearch/dependencies.js"),
                  static_url("visualsearch/visualsearch.js"),
                  static_url("generickey.js"),
                 )

            css = {
                "all": (static_url("visualsearch/visualsearch.css"),
                        static_url("hatband/css/generickey.css"),
                       )
            }

    def __init__(self, object_id_name="object_id",
                 content_type_name="content_type",
                 facet_url=None,
                 query_lookup_url=None,
                 base_lookup_url=None,
                 *args, **kwargs):
        super(GenericKeyWidget, self).__init__(*args, **kwargs)
        self.object_id_name = object_id_name
        self.content_type_name = content_type_name
        self.facet_url = facet_url
        self.query_lookup_url = query_lookup_url
        self.base_lookup_url = base_lookup_url

    def render(self, name, value, attrs=None):
        if value is None:
            value = ''
        final_attrs = self.build_attrs(attrs, name=name)
        final_attrs.update({
            "value": value,
            "is_templated": final_attrs["id"].find("__prefix__") > -1,
            "object_id_name": self.object_id_name,
            "content_type_name": self.content_type_name,
            "facet_url": self.facet_url or
                    reverse("admin:generic_key_facets"),
            "query_lookup_url": (self.query_lookup_url or
                    reverse("admin:type_and_model_to_query")),
            "base_lookup_url": (self.base_lookup_url or
                    reverse("admin:index"))

        })
        return render_to_string(self.template, final_attrs)



from django.db import models


class TestCategory(models.Model):
    name = models.CharField(max_length=255)


class TestArticle(models.Model):
    text = models.TextField()
    category = models.ForeignKey(TestCategory)



from pkgutil import extend_path
__path__ = extend_path(__path__, __name__)

# Make this a drop-in replacement for Django's built-in Admin
from django.contrib.admin.helpers import ACTION_CHECKBOX_NAME
from django.contrib.admin.options import HORIZONTAL, VERTICAL

# Below are overrides that we provide that are Hatband specific
from .options import ModelAdmin, StackedInline, TabularInline
from armstrong.hatband.sites import AdminSite, site


def autodiscover():
    """
    TODO: document
    """
    from django.contrib.admin import autodiscover as django_autodiscover
    django_autodiscover()

    from copy import copy
    from django.contrib.admin import site as django_site
    registry = copy(django_site._registry)
    registry.update(site._registry)
    site._registry = registry



from pkgutil import extend_path
__path__ = extend_path(__path__, __name__)

from django.contrib.admin.widgets import *
from .base import RichTextWidget
from .ckeditor import CKEditorWidget
from .visualsearch import GenericKeyWidget



from django.http import HttpResponse
import json


class JsonResponse(HttpResponse):
    """
    Simple HttpResponse object that takes a JSON value as it's parameter

    TODO: Find a proper home for this.
    """
    def __init__(self, data, *args, **kwargs):
        super(JsonResponse, self).__init__(json.dumps(data), *args, **kwargs)



import json
import random

from ._utils import HatbandTestCase as TestCase

from ..http import JsonResponse


class JsonResponseTestCase(TestCase):
    def test_turns_body_into_json(self):
        data = {
            "foo": "bar",
            "random": random.randint(1000, 2000),
        }

        response = JsonResponse(data)
        self.assertIsA(response.content, str, msg="sanity check")
        self.assertEqual(json.loads(response.content), data)



from django.contrib.admin.sites import AdminSite as DjangoAdminSite

from .views.search import GenericKeyFacetsMixin
from .views.search import ModelPreviewMixin
from .views.search import ModelSearchBackfillMixin
from .views.search import TypeAndModelQueryMixin

# TODO: Remove these when we reach version 2.0, these are here to maintain
#       backwards compatibility.
from .views.search import EXCLUDED_MODELS_FROM_FACETS
from .views.search import EXCLUDED_APPS_FROM_FACETS


class AdminSite(GenericKeyFacetsMixin, ModelSearchBackfillMixin,
        ModelPreviewMixin, TypeAndModelQueryMixin, DjangoAdminSite):
    pass


site = AdminSite()



import json

from django.contrib.auth.models import User
from django.contrib.contenttypes.models import ContentType
from django.core.urlresolvers import reverse
from django.test.client import Client

from ._utils import HatbandTestCase as TestCase
from .hatband_support.models import *
from .. import autodiscover

autodiscover()

PASSWORD = "password"

__all__ = [
    "GenericKeyFacetsViewTestCase",
    "TypeAndModelToQueryViewTestCase",
    "RenderModelPreviewTestCase",
    "ModelSearchBackfillMixinTestCase",
]


def staff_login(func):
    def inner(self, *args, **kwargs):
        self.client.login(username=self.staff.username, password=PASSWORD)
        return func(self, *args, **kwargs)
    return inner


def admin_login(func):
    def inner(self, *args, **kwargs):
        self.client.login(username=self.admin.username, password=PASSWORD)
        return func(self, *args, **kwargs)
    return inner


class HatbandViewTestCase(TestCase):
    def setUp(self):
        super(HatbandViewTestCase, self).setUp()
        if self.view_name:
            self.url = reverse(self.view_name)
        self.client = Client()
        self.admin = User.objects.create_superuser(
                username="test_admin",
                email="test@example.com",
                password=PASSWORD)
        self.admin.is_superuser = True
        self.admin.save()

        self.staff = User.objects.create_user(
                username="test_staff",
                email="test_staff@example.com",
                password=PASSWORD)
        self.staff.is_staff = True
        self.staff.save()

        self.regular_user = User.objects.create_user(
                username="joe_user",
                email="joe_user@example.com",
                password=PASSWORD)

    def get_response(self):
        return self.client.get(self.url)

    def assert_not_pattern_for_user(self, user, pattern):
        self.client.login(username=user.username, password=PASSWORD)
        response = self.get_response()
        self.assertNotRegexpMatches(response.content, pattern)
        self.client.logout()

    def assert_not_allowed(self, user=None):
        if user is not None:
            self.client.login(username=user.username, password=PASSWORD)
        response = self.get_response()
        self.assertRegexpMatches(response.content,
                r'<input id="id_username"')
        if user is not None:
            self.client.logout()

    def test_requires_staff_user(self):
        self.assert_not_allowed(user=None)

        pattern = r'<input id="id_username"'
        self.assert_not_pattern_for_user(self.admin, pattern)
        self.assert_not_pattern_for_user(self.staff, pattern)

        self.regular_user.is_staff = True
        self.assert_not_allowed(self.regular_user)


class GenericKeyFacetsViewTestCase(HatbandViewTestCase):
    view_name = "admin:generic_key_facets"


class ModelSearchBackfillMixinTestCase(HatbandViewTestCase):
    view_name_template = "admin:%s_%s_search"
    view_name = False

    def get_response(self):
        return self.client.get(self.url, {"q": self.content_type.model})

    def setUp(self):
        super(ModelSearchBackfillMixinTestCase, self).setUp()
        self.content_type = ContentType.objects.get(app_label="auth",
                model="user")
        view_name = self.view_name_template % (
                self.content_type.app_label,
                self.content_type.model)
        self.url = reverse(view_name)

    @admin_login
    def test_search_returns_json(self):
        response = self.get_response()
        data = json.loads(response.content)
        self.assertEqual(1, len(data["results"]))
        expected = "%s: %s" % (self.regular_user.pk,
                self.regular_user.username)
        self.assertEqual(expected, data["results"][0])


class TypeAndModelToQueryViewTestCase(HatbandViewTestCase):
    view_name = "admin:type_and_model_to_query"

    def get_response(self):
        content_type = ContentType.objects.get_for_model(User)
        obj = content_type.model_class().objects.all()[0]
        return self.client.get(self.url, {
                "content_type_id": content_type.pk,
                "object_id": obj.pk})

    @staff_login
    def test_query_is_inside_response(self):
        response = self.get_response()
        data = json.loads(response.content)
        self.assertTrue("query" in data)

    @staff_login
    def test_bad_response_on_unknown_content_type(self):
        response = self.client.get(self.url, {"content_type_id": -1})
        self.assertEqual(400, response.status_code)

    @staff_login
    def test_bad_response_on_no_object_id(self):
        content_type = ContentType.objects.all()[0]
        response = self.client.get(self.url,
                {"content_type_id": content_type.pk})
        self.assertEqual(400, response.status_code)

    @staff_login
    def test_returns_empty_query_on_unknown_object_id(self):
        content_type = ContentType.objects.all()[0]
        response = self.client.get(self.url,
                {"content_type_id": content_type.pk, "object_id": -1})
        self.assertEqual(200, response.status_code)

        data = json.loads(response.content)
        self.assertTrue("query" in data)
        self.assertEqual("", data["query"])

    @staff_login
    def test_returns_string_suitable_for_display_in_gfk_widget(self):
        content_type = ContentType.objects.get_for_model(User)
        obj = content_type.model_class().objects.all()[0]
        response = self.client.get(self.url, {
                "content_type_id": content_type.pk,
                "object_id": obj.pk})
        data = json.loads(response.content)
        query = data["query"]
        self.assertEqual('%s: "%d: %s"' % (content_type.model, obj.pk, obj, ),
                         query)


class RenderModelPreviewTestCase(HatbandViewTestCase):
    view_name = "admin:render_model_preview"

    def get_response(self):
        cat = TestCategory.objects.create(name="cat")
        TestArticle.objects.create(text="Hooray", category=cat)
        content_type = ContentType.objects.get_for_model(TestArticle)
        obj = content_type.model_class().objects.all()[0]
        return self.client.get(self.url, {
                "content_type": content_type.pk,
                "object_id": obj.pk})

    def test_template_rendered(self):
        self.client.login(username=self.staff.username, password=PASSWORD)
        response = self.get_response()
        self.assertEqual(response.content, 'It worked! Hooray\n')



from django.contrib import admin
from django.contrib.admin.options import InlineModelAdmin
from django.utils.safestring import mark_safe

from . import forms


class ModelAdmin(admin.ModelAdmin):
    formfield_overrides = forms.RICH_TEXT_DBFIELD_OVERRIDES


class StackedInline(admin.StackedInline):
    formfield_overrides = forms.RICH_TEXT_DBFIELD_OVERRIDES


class TabularInline(admin.TabularInline):
    formfield_overrides = forms.RICH_TEXT_DBFIELD_OVERRIDES


class GenericKeyInline(InlineModelAdmin):
    form = forms.OrderableGenericKeyLookupForm
    formfield_overrides = forms.RICH_TEXT_DBFIELD_OVERRIDES
    template = "admin/edit_inline/generickey.html"


class BackboneInline(InlineModelAdmin):
    form = forms.OrderableGenericKeyLookupForm
    formfield_overrides = forms.RICH_TEXT_DBFIELD_OVERRIDES
    template = "admin/edit_inline/backbone.html"
    extra = 0



from django.forms import widgets
from django.conf import settings


class CKEditorWidget(widgets.Textarea):

    class Media:
        js = (''.join((settings.STATIC_URL, "ckeditor/ckeditor.js")),)

    def __init__(self, attrs=None):
        final_attrs = {'class': 'ckeditor'}
        if attrs is not None:
            final_attrs.update(attrs)
            if 'class' in attrs:
                final_attrs['class'] = ' '.join((attrs['class'], 'ckeditor'))
        super(CKEditorWidget, self).__init__(attrs=final_attrs)



from armstrong.dev.tests.utils import ArmstrongTestCase
import random


def random_range():
    # TODO: make sure this can only be generated once
    return range(random.randint(1000, 2000))


class HatbandTestCase(ArmstrongTestCase):
    pass


class HatbandTestMixin(object):

    script_code = """
    <script type="text/javascript" src="/static/ckeditor/ckeditor.js"></script>
    """.strip()
    textarea_code = 'class="ckeditor"></textarea>'

    def assertCkEditorPresent(self, response):
        self.assertContains(response, self.script_code)
        self.assertContains(response, self.textarea_code)

    def assertCkEditorNotPresent(self, response):
        self.assertNotContains(response, self.script_code)
        self.assertNotContains(response, self.textarea_code)



from pkgutil import extend_path
__path__ = extend_path(__path__, __name__)

from contextlib import contextmanager
import fudge

from .http import *
from .sites import *
from .widgets import *

from ... import hatband


def generate_random_registry():
    from ._utils import random_range
    return dict([("key%d" % i, i) for i in random_range()])


@contextmanager
def fake_autodiscover():
    from django.contrib import admin
    autodiscover = fudge.Fake().is_callable()
    with fudge.patched_context(admin, "autodiscover", autodiscover):
        yield

@contextmanager
def fake_django_site_registry(test):
    with fake_autodiscover():
        random_registry = generate_random_registry()
        from django.contrib import admin
        site = fudge.Fake()
        site.has_attr(_registry=random_registry)
        with fudge.patched_context(admin, "site", site):
            test.assertEqual( len(random_registry.items()),
                    len(site._registry.items()), msg="sanity check")
            yield random_registry


class AutodiscoverTestCase(HatbandTestCase):
    def setUp(self):
        from copy import copy
        self.original_site = copy(hatband.site)
        hatband.site._registry = {}

    def tearDown(self):
        hatband.site = self.original_site

    @fudge.test
    def test_dispatches_to_djangos_autodiscover(self):
        from django.contrib import admin
        autodiscover = fudge.Fake().is_callable().expects_call()

        with fudge.patched_context(admin, "autodiscover", autodiscover):
            hatband.autodiscover()


    @fudge.test
    def test_has_a_copy_of_main_django_registry(self):
        random_registry = generate_random_registry()

        from django.contrib import admin
        site = fudge.Fake()
        site.has_attr(_registry=random_registry)

        with fake_autodiscover():
            with fudge.patched_context(admin, "site", site):
                hatband.autodiscover()
                for key in random_registry.keys():
                    self.assertTrue(key in hatband.site._registry)


    @fudge.test
    def test_has_hatband_registered_plus_(self):
        with fake_django_site_registry(self) as random_registry:
            from .hatband_support.models import TestCategory

            self.assertFalse(TestCategory in hatband.site._registry.keys(),
                    msg="Sanity check")
            hatband.site.register(TestCategory)
            self.assertTrue(TestCategory in hatband.site._registry.keys(),
                    msg="Sanity check")

            hatband.autodiscover()
            registry = hatband.site._registry.items()
            self.assertTrue(TestCategory in hatband.site._registry.keys(),
                    msg="TestCategory should still be in the registry")

    @fudge.test
    def test_original_django_sites_registry_remains_untouched(self):
        with fake_django_site_registry(self) as random_registry:
            from .hatband_support.models import TestCategory
            self.assertFalse(TestCategory in random_registry.keys())
            hatband.site.register(TestCategory)
            hatband.autodiscover()

            self.assertFalse(TestCategory in random_registry.keys())



from .http import JsonResponse


def json_response(func):
    """
    Wrap the return value as a JsonResponse object

    Prefered method is to use JsonResponse directly now.

    .. deprecated:: 1.2
    """
    def inner(*args, **kwargs):
        data = func(*args, **kwargs)
        return JsonResponse(data)
    return inner
